/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.myfaces.shared_tomahawk.util.servlet;

import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;

import javax.faces.context.FacesContext;
import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

public class SourceCodeServlet extends HttpServlet {

	public static HttpSession currentContext() {
		return (HttpSession)FacesContext.getCurrentInstance()
		.getExternalContext().getSession(true);
	}
	
	private String getJavaSourceDirectoryFromValueBinding(){
		FacesContext facesContext = FacesContext.getCurrentInstance();
		return (String)facesContext.getApplication().createValueBinding("#{javaSourceDirectory}").getValue(facesContext);		
	}
	
	private String getJavaSourceDirectoryFromServletParam(){
	  return (String)getServletConfig().getInitParameter("source.dir");
	}

	private String getServerPath(String dir, String wp) {
		String javapath = wp.replace('.', '/') + ".java";
		String p = "/WEB-INF/" + dir + "/" + javapath;
		String realPath = getServletConfig().getServletContext().getRealPath(p);
		String result = realPath.replace("\\.\\", "\\");
		return result;
	}


	private String getAbsolutePath(String dir, String wp) {
		String javapath = wp.replace('.', '\\') + ".java";
		String result = dir + javapath;
		return result;
	}

	public void doGet(HttpServletRequest req, HttpServletResponse res)
			throws IOException, ServletException {
		String webPage = req.getServletPath();

		// remove the '*.source' suffix that maps to this servlet
		int chopPoint = webPage.lastIndexOf(".source");

		webPage = webPage.substring(0, chopPoint); // /org.apache.myfaces.examples.tree.TreeBacker.java

		if (webPage.endsWith(".jsf")) {
			int jsfChopPoint = webPage.lastIndexOf(".jsf");

			webPage = webPage.substring(0, jsfChopPoint);

			webPage += ".jsp"; // replace jsf with jsp

			// get the actual file location of the requested resource
			String realPath = getServletConfig().getServletContext()
					.getRealPath(webPage);

			outputFile(res, realPath);
		} else if (webPage.endsWith(".jsp")) {
			// get the actual file location of the requested resource
			String realPath = getServletConfig().getServletContext()
					.getRealPath(webPage);

			outputFile(res, realPath);
		} else if (webPage.endsWith(".jspx")) {
			// get the actual file location of the requested resource
			String realPath = getServletConfig().getServletContext()
					.getRealPath(webPage);

			outputFile(res, realPath);
		} else if (webPage.endsWith(".xhtml")) {
			// get the actual file location of the requested resource
			String realPath = getServletConfig().getServletContext()
					.getRealPath(webPage);
			outputFile(res, realPath);
		} else {
			int beginChopPoint = webPage.lastIndexOf("/");
			int extensionChopPoint = webPage.lastIndexOf(".java");
			webPage = webPage.substring(beginChopPoint + 1, extensionChopPoint);
			try {
				outputFile(res, getAbsolutePath(getJavaSourceDirectoryFromServletParam(),webPage));	
			} catch (Exception e) {
				try {
					 outputFile(res, getServerPath("classes", webPage));
				} catch (Exception e1) {
					try {
						 outputFile(res, getServerPath("src", webPage));
					} catch (Exception e2) {
						 outputString(res, "source directory not found, update servlet init param (source.dir) !");
					}	
				}
			}
		}
	}

	private void outputString(HttpServletResponse res, String mesg) throws IOException {
		res.setContentType("text/plain");
		ServletOutputStream out = res.getOutputStream();
		out.println(mesg);
	}

	private void outputFile(HttpServletResponse res, String realPath)
			throws IOException {
		// output an HTML page
		res.setContentType("text/plain");

		// print some html
		ServletOutputStream out = res.getOutputStream();

		// print the file
		InputStream in = null;
		try {
			in = new BufferedInputStream(new FileInputStream(realPath));
			int ch;
			while ((ch = in.read()) != -1) {
				out.print((char) ch);
			}
		} finally {
			if (in != null)
				in.close(); // very important
		}
	}
}
